"use strict";
// Most of this code was taken from @satya164's babel-plugin-css-prop
// and styled-components babel plugin
// @see https://github.com/satya164/babel-plugin-css-prop
// @see https://github.com/styled-components/babel-plugin-styled-components
Object.defineProperty(exports, "__esModule", { value: true });
const util_1 = require("./util");
const TAG_NAME_REGEXP = /^[a-z][a-z\d]*(\-[a-z][a-z\d]*)?$/;
function getVisitorsPreprocessCSS({ types: t }, options) {
    return (path, state) => {
        const tag = path.node.name.name;
        if (tag !== 'css') {
            return;
        }
        console.log(state.file.opts.filename, state.importLocalNames.styled);
        const styledIdentifier = util_1.assureImport({ types: t }, state, 'styled', 'twstyled');
        const program = state.file.path;
        const { bindings } = program.scope;
        let css;
        if (t.isStringLiteral(path.node.value)) {
            css = t.templateLiteral([
                t.templateElement({ raw: path.node.value.value, cooked: path.node.value.value }, true)
            ], []);
        }
        else if (t.isJSXExpressionContainer(path.node.value)) {
            if (t.isTemplateLiteral(path.node.value.expression)) {
                css = path.node.value.expression;
            }
            else if (t.isTaggedTemplateExpression(path.node.value.expression) &&
                t.isIdentifier(path.node.value.expression.tag) &&
                path.node.value.expression.tag.name === 'css') {
                css = path.node.value.expression.quasi;
            }
            else if (t.isObjectExpression(path.node.value.expression)) {
                css = path.node.value.expression;
            }
            else if (t.isJSXEmptyExpression(path.node.value.expression)) {
                /** noop */
            }
            else
                css = t.templateLiteral([
                    t.templateElement({ raw: '', cooked: '' }, false),
                    t.templateElement({ raw: '', cooked: '' }, true)
                ], [path.node.value.expression]);
        }
        if (!css)
            return;
        const elem = path.parentPath;
        const name = getName(elem.node.name, t);
        const nameExpression = getNameExpression(elem.node.name, t);
        const id = path.scope.generateUidIdentifier('Styled' + name.replace(/^([a-z])/, (match, p1) => p1.toUpperCase()));
        let styled;
        let injector;
        if (TAG_NAME_REGEXP.test(name)) {
            styled = t.callExpression(styledIdentifier, [t.stringLiteral(name)]);
        }
        else {
            styled = t.callExpression(styledIdentifier, [nameExpression]);
            if (bindings[name] &&
                !t.isImportDeclaration(bindings[name].path.parent)) {
                injector = (nodeToInsert) => (t.isVariableDeclaration(bindings[name].path.parent)
                    ? bindings[name].path.parentPath
                    : bindings[name].path).insertAfter(nodeToInsert);
            }
        }
        elem.node.attributes = elem.node.attributes.filter((attr) => attr !== path.node);
        elem.node.name = t.jSXIdentifier(id.name);
        if (elem.parentPath.node.closingElement) {
            ;
            elem.parentPath
                .node.closingElement.name = t.jSXIdentifier(id.name);
        }
        if (t.isObjectExpression(css)) {
            // object syntax
            throw new Error('object syntax is not yet supported in css attributes');
        }
        else {
            // tagged template literal
            css.expressions = css.expressions.reduce((acc, ex) => {
                if (Object.keys(bindings).some((key) => bindings[key].referencePaths.find((p) => p.node === ex)) ||
                    t.isFunctionExpression(ex) ||
                    t.isArrowFunctionExpression(ex)) {
                    acc.push(ex);
                }
                else {
                    const name = path.scope.generateUidIdentifier('css');
                    const p = t.identifier('p');
                    elem.node.attributes.push(t.jSXAttribute(t.jSXIdentifier(name.name), t.jSXExpressionContainer(ex)));
                    const fn = t.arrowFunctionExpression([p], t.memberExpression(p, name));
                    fn.loc = path.node.loc;
                    acc.push(fn);
                }
                return acc;
            }, []);
        }
        if (!injector) {
            let parent = elem;
            while (!t.isProgram(parent.parentPath)) {
                parent = parent.parentPath;
            }
            injector = (nodeToInsert) => parent.insertBefore(nodeToInsert);
        }
        injector(t.variableDeclaration('var', [
            t.variableDeclarator(id, t.isObjectExpression(css) || t.isArrowFunctionExpression(css)
                ? t.callExpression(styled, [css])
                : t.taggedTemplateExpression(styled, css))
        ]));
    };
}
exports.default = getVisitorsPreprocessCSS;
function getName(node, t) {
    if (t.isJSXMemberExpression(node)) {
        return `${getName(node.object, t)}.${node.property.name}`;
    }
    if (typeof node.name === 'string') {
        return node.name;
    }
    throw new Error(`Cannot infer name from node with type "${node.type}". `);
}
function getNameExpression(node, t) {
    if (t.isJSXMemberExpression(node)) {
        return t.memberExpression(getNameExpression(node.object, t), t.identifier(node.property.name));
    }
    if (typeof node.name === 'string') {
        return t.identifier(node.name);
    }
    throw new Error(`Cannot infer name expression from node with type "${node.type}". `);
}
//# sourceMappingURL=visitor-preprocessor-css.js.map